/***********************************
 *        Timothy Hannafin         *
 *        hannafit@wpi.edu         *
 * Worcester Polytechnic Institute *
 *               2006              *
 * *********************************/

using System;
using System.Net;
using System.Net.Sockets;
using System.Data;
using System.Text;
using System.IO;
using System.Timers;


namespace RobotController
{
	/// <summary>
	/// The command server is resposible for getting input from the client in the form of short strings 
	/// and executing those commands. Coomands include movement control and turning autonomous mode on and off
	/// </summary>
	public class CommandServer
	{
		private const float COMMAND_LIFETIME = 1000; //the lifetime in ms of an incremental command

		private System.Timers.Timer serverSensorTimer = new System.Timers.Timer();

		private string address; 
		private int port;
		private bool exitServerFlag = true;
		private StreamReader reader;
		private Robot myRobot;
		private Controller form1;
		private float speed;

		private AutonoDriver aDriver;
		private string movementType;

		public CommandServer(string addr, int prt, Robot arobot, Controller ctrl)
		{
			address = addr;
			port = prt;
			myRobot = arobot;
			form1 = ctrl;
			speed = form1.speed;
			aDriver = new AutonoDriver(myRobot, speed);
		}

		/// <summary>
		/// Start the command server and wait for a new connection
		/// </summary>
		public void Start()
		{

			IPAddress addr = IPAddress.Parse(address);
			TcpListener listen;

			TcpClient client;
			NetworkStream clientStream;
			
			string line;

			serverSensorTimer.Elapsed += new System.Timers.ElapsedEventHandler(sensorTimer);
			serverSensorTimer.Interval = 10;
			bool connected = true;
				


			while (exitServerFlag)
			{
				listen = new TcpListener(addr, port);
				listen.Start();

				client = listen.AcceptTcpClient();
				clientStream = client.GetStream();

				StatusType oldStatus = myRobot.Status;

				myRobot.Status = StatusType.user_control;

				reader = new StreamReader(clientStream, Encoding.UTF8);
				connected = true;

				while(connected && exitServerFlag)
				{
						
					if (listen != null)
					{
						try
						{
							line = reader.ReadLine();  //get the movement type code
						}
						catch  //this will happen when the client disconnects unexpectedly (crashes) this will just send it back to the top of the loop
						{
							line = "z";
							connected = false;
						}
						serverSensorTimer.Start();

						switch (line)
						{
							case "new":  //new client connected
								addStatus("New client has connected from " + reader.ReadLine()); //new command sends in the form of "new\n[clientIP]\n"
								break;

							case "ext": //client disconnect
								addStatus("Client has disconnected");
								connected = false;
								myRobot.Status = oldStatus;
								break;

							case "spd":  //change speed
								line = reader.ReadLine();
								speed =  Convert.ToSingle(line);
								form1.txtSpeed.Text = line;
								break;

							case "stp":  //stop
								myRobot.Status = StatusType.user_control;
								myRobot.stop();
								break;

							case "inc":  //incemental command
								line = reader.ReadLine();  //get command
								incrementalCommands(line); //execute the incremental command
								break;

							case "con":  //continuous command
								line = reader.ReadLine();  //get command
								continuousCommands(line);  //execute the continuous command
								break;

							case "auto":  //autonomus mode
								myRobot.Status = StatusType.navigating;  //change the robot status
								aDriver.Start();  //start the AutonoDriver (if already started this will do nothing)
								break;
						}
					}
				}// client connected while

				//when client disconnects close all the connections and restart them at the top of the loop
				reader.Close();
				client.Close();
				listen.Stop();

			}

			
			
		}

		/// <summary>
		/// Stop the command server
		/// </summary>
		public void Stop()
		{
			reader.Close();
			exitServerFlag = false;
		}

		/// <summary>
		/// Execute a continuous command
		/// </summary>
		/// <param name="line"></param>
		private void continuousCommands(string command)
		{
			movementType = command;
			switch (command)
			{
				case "fwd":

					if(myRobot.getFrontSensorValue() < 300)
						myRobot.setFowardSpeed(speed);
					addStatus("Move Foward");
					break;

				case "bak":
					myRobot.setReverseSpeed(speed);
					addStatus("Move Backwards");
					break;

				case "lef":
					if(myRobot.getLeftSensorValue() < 300)
					{
						myRobot.turnLeft(speed);
					}
					addStatus("Turn Left");
					break;

				case "rit":
					if(myRobot.getRightSensorValue() < 300)
					{
						myRobot.turnRight(speed);
					}
					addStatus("Turn Right");
					break;
			}
		}

		/// <summary>
		/// Execute an inceremental command
		/// </summary>
		/// <param name="command"></param>
		private void incrementalCommands(string command)
		{
			movementType = command;
			switch (command)
			{
				case "fwd":

					if(myRobot.getFrontSensorValue() < 300)
						myRobot.setFowardSpeed(speed, COMMAND_LIFETIME);
					addStatus("Move Foward");
					break;

				case "bak":
					myRobot.setReverseSpeed(speed, COMMAND_LIFETIME);
					addStatus("Move Backwards");
					break;

				case "lef":
					if(myRobot.getLeftSensorValue() < 300)
					{
						myRobot.turnLeft(speed, COMMAND_LIFETIME);
					}
					addStatus("Turn Left");
					break;

				case "rit":
					if(myRobot.getRightSensorValue() < 300)
					{
						myRobot.turnRight(speed, COMMAND_LIFETIME);
					}
					addStatus("Turn Right");
					break;
			}

		}

		/// <summary>
		/// Add a status message to the main form
		/// </summary>
		/// <param name="message"></param>
		private void addStatus(string message)
		{
			form1.txtStatus.AppendText(message + System.Environment.NewLine);

		}

		/// <summary>
		/// Sensor reader to make sure the robot doesnt crash while under user control
		/// </summary>
		/// <param name="source"></param>
		/// <param name="e"></param>
		private void sensorTimer(object source, ElapsedEventArgs e)
		{
			if(myRobot.getFrontSensorValue() > 300 && movementType == "fwd")
			{
				serverSensorTimer.Stop();
				myRobot.stop();
			}
			if(myRobot.getLeftSensorValue() > 300 && movementType == "lef")
			{
				serverSensorTimer.Stop();
				myRobot.stop();
			}
			if(myRobot.getRightSensorValue() > 300 && movementType == "rit")
			{
				serverSensorTimer.Stop();
				myRobot.stop();
			}
		}
	}


}
