/***********************************
 *        Timothy Hannafin         *
 *        hannafit@wpi.edu         *
 * Worcester Polytechnic Institute *
 *               2006              *
 * *********************************/

using System;
using System.IO;
using System.Timers;
using System.Threading;

namespace RobotController
{
	/// <summary>
	/// Script reads in an external script into an array of ScriptCommands
	/// When the ExecuteNextBlock function is called, Script takes over command of
	/// the robot and exeuctes the block of commands at the current index
	/// </summary>
	public class Script
	{
		Robot myRobot;
		ScriptCommand[] commands;
		int index, repindex, numreps, iterations;
		float defSpeed;  //the default speed of the robot
		public bool executing;

		public Script(string filename, Robot aRobot, float spd)
		{
			executing = false;
			index = 0;
			myRobot = aRobot;
			commands = new ScriptCommand[128];
			defSpeed = spd;
			readFile(filename);
		}

		/// <summary>
		/// Parsees the script file in the working directory into an array of commands
		/// </summary>
		private void readFile(string filename)
		{
			string line = "";
			string[] strings = new string[4];
			StreamReader file = new StreamReader(filename);
			line = file.ReadLine();
			while(line != null)
			{
				commands[index] = new ScriptCommand();
				strings = line.Split(',');

				switch(strings[0])
				{
					case "STARTBLOCK":
						addStartblock(strings[1]);
						break;

					case "ENDBLOCK":
						addEndblock();
						break;

					case "LEFT":
						addLeft(strings[1], strings[2]);
						break;

					case "RIGHT":
						addRight(strings[1], strings[2]);
						break;

					case "STRAIGHT":
						addStraight(strings[1], strings[2]);
						break;

					case "REVERSE":
						addReverse(strings[1], strings[2]);
						break;

					case "REP":
						addRep(strings[1]);
						break;

					case "ENDREP":
						addEndRep();
						break;
				}
				line = file.ReadLine();
			}

			file.Close();
			index=0;
		}
        
		#region Parsing Commands
		private void addStartblock(string name)
		{
			commands[index].type = "STARTBLOCK";
			commands[index].name = name;
			index++;

		}

		private void addEndblock()
		{
			commands[index].type = "ENDBLOCK";
			index++;

		}

		private void addLeft(string duration, string speed)
		{
			commands[index].type = "LEFT";
			if(speed == "DEF")
			{
				commands[index].speed = defSpeed;
			}
			else
			{
				commands[index].speed = Convert.ToSingle(speed);
			}
			commands[index].duration = Convert.ToSingle(duration)*1000;
			index++;
		}

		private void addRight(string duration, string speed)
		{
			commands[index].type = "RIGHT";
			if(speed == "DEF")
			{
				commands[index].speed = defSpeed;
			}
			else
			{
				commands[index].speed = Convert.ToSingle(speed);
			}
			commands[index].duration = Convert.ToSingle(duration)*1000;
			index++;
		}

		private void addStraight(string duration, string speed)
		{
			commands[index].type = "STRAIGHT";
			if(speed == "DEF")
			{
				commands[index].speed = defSpeed;
			}
			else
			{
				commands[index].speed = Convert.ToSingle(speed);
			}
			commands[index].duration = Convert.ToSingle(duration)*1000;
			index++;
		}

		private void addReverse(string duration, string speed)
		{
			commands[index].type = "REVERSE";
			if(speed == "DEF")
			{
				commands[index].speed = defSpeed;
			}
			else
			{
				commands[index].speed = Convert.ToSingle(speed);
			}
			commands[index].duration = Convert.ToSingle(duration)*1000;
			index++;
		}

		private void addRep(string numReps)
		{
			commands[index].type = "REP";
			commands[index].iterations = Convert.ToInt16(numReps);
			index++;
		}

		private void addEndRep()
		{
			commands[index].type = "ENDREP";
			index++;
		}
		#endregion

		/// <summary>
		/// Execute the next block of commands
		/// </summary>
		public void executeNextBlock()
		{
			myRobot.Status = StatusType.executing_block;

			string returnString = "";
			while(commands[index] != null)
			{
				if(myRobot.Status == StatusType.executing_block)
				{
					switch (commands[index].type)
					{
						case "STARTBLOCK":
							Console.WriteLine(commands[index].name);
							myRobot.Status = StatusType.executing_block;
							returnString = commands[index].name;
							break;

						case "ENDBLOCK":
							myRobot.Status = StatusType.navigating;
							index++;
							return;
							break;

						case "LEFT":
							myRobot.turnLeft(commands[index].speed, commands[index].duration);
							break;

						case "RIGHT":
							myRobot.turnRight(commands[index].speed, commands[index].duration);
							break;

						case "STRAIGHT":
							myRobot.setFowardSpeed(commands[index].speed, commands[index].duration);
							break;

						case "REVERSE":
							myRobot.setReverseSpeed(commands[index].speed, commands[index].duration);
							break;

						case "REP":  //if this is a rep
							repindex = index;  //store this index to return to later
							iterations = commands[index].iterations;
							break;

						case "ENDREP":
							if(iterations != 0) //use 0 to indicate infinite loop
								numreps++;

							if(numreps <= iterations)  //if we havnt done this loop iterations times yet
								index = repindex;  //go back to the top of the loop
							break;

					}
					index++;
				}
				if(myRobot.Status == StatusType.navigation_error || myRobot.Status == StatusType.idle)
				{
					return;
				}
			}

		}

		/// <summary>
		/// Skip to the end of the currently executing block or next block in queue
		/// </summary>
		public void SkipToEndOfBlock()
		{
			while(commands[index].type != "ENDBLOCK")
				index++;
		}
	}
}
