/***********************************
 *        Timothy Hannafin         *
 *        hannafit@wpi.edu         *
 * Worcester Polytechnic Institute *
 *               2006              *
 * *********************************/

using System;
using System.Drawing;
using System.IO;

namespace RobotController
{
	/// <summary>
	/// WebcamInterface provides a wrapper for an open source webcam package
	/// it also adds functionality that converts captured frames into formats 
	/// suitable for transmission to the remote client and for analysis by the
	/// neural network
	/// </summary>
	
	public class WebcamInterface
	{
		private const int NN_HEIGHT = 45;  //dimensions of the image the neural net is designed to take
		private const int NN_WIDTH = 60;

		private const int NUM_OF_BUFFERS = 4;  //the number of buffers in the webcam video stream
		private WebCam_Capture.WebCamCapture wc;
		private Image[] images;
		private static int imageIndexw;  //image index to read from
		private static int imageIndexr;  //image index to write to



		private Controller form1;

		public WebcamInterface(Controller frm)
		{
			form1 = frm;
			wc = new WebCam_Capture.WebCamCapture();

			wc.ImageCaptured += new WebCam_Capture.WebCamCapture.WebCamEventHandler(WebCamCapture_ImageCaptured);
			wc.Height = form1.picVideo.Height;
			wc.Width = form1.picVideo.Width;
			wc.TimeToCapture_milliseconds = 50;

			images = new Image[4];
			imageIndexw = 0;
			imageIndexr =-1;
			
		}

		
		/// <summary>
		/// This function is called every time the webcam captures a new frame
		/// </summary>
		/// <param name="source"></param>
		/// <param name="e"></param>
		private void WebCamCapture_ImageCaptured(object source, WebCam_Capture.WebcamEventArgs e)
		{
			form1.picVideo.Image = e.WebCamImage;  //display this frame on the main form
			images[imageIndexw] = (Image) e.WebCamImage.Clone();  //put this frame in the image buffer
			

			//loop through the buffer
			if(imageIndexw == NUM_OF_BUFFERS-1)
			{
				imageIndexw = 0;
				imageIndexr++;
			}
			else if(imageIndexw == 0)
			{
				imageIndexw++;
				imageIndexr = 0;
			}
			else
			{
				imageIndexw++;
				imageIndexr++;
			}

		}

		/// <summary>
		/// Gets the latest fully buffered frame in a format for sending
		/// </summary>
		public byte[] getWebImage()
		{

			byte[] img;

			lock(images)
			{
				MemoryStream ms = new MemoryStream();
				images[imageIndexr].Save(ms, System.Drawing.Imaging.ImageFormat.Jpeg);
				img = ms.ToArray();
			}

			return img;

		}

		/// <summary>
		/// returns a float array representation of the latest frame in a format for the neural net
		/// </summary>
		public float[] getNeuralNetImage()
		{
			return  prepForNeuralNet(images[imageIndexr]);

		}

		/// <summary>
		/// start the camera
		/// </summary>
		public void start()
		{
			wc.Start(0);
		}

		/// <summary>
		/// stop the camera
		/// </summary>
		public void stop()
		{
			wc.Stop();
		}

		/// <summary>
		/// Prepares the latest frame for the neural net by converting it into a float array
		/// </summary>
		private float[] prepForNeuralNet(Image img)
		{
			//resize to the dimensions expected by the neural net
			img = img.GetThumbnailImage(NN_HEIGHT, NN_WIDTH,  new System.Drawing.Image.GetThumbnailImageAbort(ThumbnailCallback), IntPtr.Zero);
			//convert to bitmap for easy manipulation
			Bitmap newImage = new Bitmap(img);
			float[] flt = new float[newImage.Height*newImage.Width];
			int i =0;
			int greyVal;

			for(int x=0; x<newImage.Width; x++)  //add the RGB values of each pixel together then put that in the float array
				for(int y=0; y<newImage.Height; y++)
				{
					greyVal = newImage.GetPixel(x,y).R + newImage.GetPixel(x,y).B + newImage.GetPixel(x,y).G;
					flt[i] = greyVal;
					i++;
				}
			return flt;

		}

		/// <summary>
		/// Needed for GetThumbnailImage
		/// </summary>
		public bool ThumbnailCallback()
		{
			return true;
		}


	}
}
